@@ -41,7 +41,7 @@ import pyglet
 # # Default to OpenAL if available:
 # pyglet.options['audio'] = 'openal', 'pulse', 'directsound', 'silent'
 
-from pyglet.gl import *
+from pyglet.gl import glClearColor
 from pyglet.graphics import Group
 from pyglet.math import Mat4, Vec3
 
@@ -79,12 +79,12 @@ class Handle:
         self.player = player
 
     def hit_test(self, x, y, z):
-        dx, dy, dz = [a - b for a, b in zip(self.pos(), (x, y, z))]
+        dx, dy, dz = (a - b for a, b in zip(self.pos(), (x, y, z)))
         if dx * dx + dy * dy + dz * dz < self.radius * self.radius:
             return -dx, -dy, -dz
 
     def pos(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def delete(self):
         pass
@@ -240,7 +240,7 @@ class ConeAngleHandle(Handle):
         return px + x, py, pz + z
 
     def get_angle(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def update_shapes(self):
         px, _, py = self.player.position
@@ -335,7 +335,7 @@ class MoreHandle(Handle):
         return x + 1, y, z + 1
 
     def get_angle(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def update_shapes(self):
         self._box.visible = self.open
@@ -353,6 +353,7 @@ class MoreHandle(Handle):
         else:
             self._circ.position = x, z
             self._outline.position = x, z
+            self._outline.thickness = 1.0 / self.win.zoom
             r = self.radius - 0.1
             self._line0._width = 1.0 / self.win.zoom
             _update_line(self._line0, x - r, z, x + r, z)
