@@ -1,304 +1,35 @@
-# TODO Windows Vista: need to call SetProcessDPIAware?  May affect GDI+ calls as well as font.
+# Windows Vista: need to call SetProcessDPIAware?  May affect GDI+ calls as well as font.
+from __future__ import annotations
 
+import ctypes
 import math
 import warnings
-
-from sys import byteorder
-from typing import Optional
+from typing import TYPE_CHECKING, ClassVar, Sequence
 
 import pyglet
-from pyglet.font import base
-from pyglet.font import win32query
 import pyglet.image
-from pyglet.libs.win32.constants import *
+from pyglet.font import base
+from pyglet.image.codecs.gdiplus import BitmapData, ImageLockModeRead, PixelFormat32bppARGB, Rect, gdiplus
+from pyglet.libs.win32 import _gdi32 as gdi32
+from pyglet.libs.win32 import _user32 as user32
+from pyglet.libs.win32.constants import ANTIALIASED_QUALITY, FW_BOLD, FW_NORMAL
 from pyglet.libs.win32.context_managers import device_context
-from pyglet.libs.win32.types import *
-from pyglet.libs.win32 import _gdi32 as gdi32, _user32 as user32
-from pyglet.libs.win32 import _kernel32 as kernel32
-from pyglet.util import asbytes
+from pyglet.libs.win32.types import ABC, BYTE, LOGFONTW, TEXTMETRIC
 
-_debug_font = pyglet.options['debug_font']
+if TYPE_CHECKING:
+    from pyglet.font.base import Glyph
 
 
-def str_ucs2(text):
-    if byteorder == 'big':
-        text = text.encode('utf_16_be')
-    else:
-        text = text.encode('utf_16_le')   # explicit endian avoids BOM
-    return create_string_buffer(text + '\0')
-
-
-_debug_dir = 'debug_font'
-
-
-def _debug_filename(base, extension):
-    import os
-    if not os.path.exists(_debug_dir):
-        os.makedirs(_debug_dir)
-    name = f'{os.path.join(_debug_dir, base)}-{{0}}.{{1}}'
-    num = 1
-    while os.path.exists(name.format(num, extension)):
-        num += 1
-    return name.format(num, extension)
-
-
-def _debug_image(image, name):
-    filename = _debug_filename(name, 'png')
-    image.save(filename)
-    _debug(f'Saved image {image} to {filename}')
-
-
-_debug_logfile = None
-
-
-def _debug(msg):
-    global _debug_logfile
-    if not _debug_logfile:
-        _debug_logfile = open(_debug_filename('log', 'txt'), 'wt')
-    _debug_logfile.write(msg + '\n')
-
-
-class Win32GlyphRenderer(base.GlyphRenderer):
-
-    def __init__(self, font):
-        self._bitmap = None
-        self._dc = None
-        self._bitmap_rect = None
-        super(Win32GlyphRenderer, self).__init__(font)
-        self.font = font
-
-        # Pessimistically round up width and height to 4 byte alignment
-        width = font.max_glyph_width
-        height = font.ascent - font.descent
-        width = (width | 0x3) + 1
-        height = (height | 0x3) + 1
-        self._create_bitmap(width, height)
-
-        gdi32.SelectObject(self._dc, self.font.hfont)
-
-    def _create_bitmap(self, width, height) -> None:
-        pass
-
-    def render(self, text: str) -> pyglet.font.base.Glyph:
-        raise NotImplementedError('abstract')
-
-
-class GDIGlyphRenderer(Win32GlyphRenderer):
-
-    def __del__(self):
-        try:
-            if self._dc:
-                gdi32.DeleteDC(self._dc)
-            if self._bitmap:
-                gdi32.DeleteObject(self._bitmap)
-        except:
-            pass
-
-    def render(self, text: str) -> pyglet.font.base.Glyph:
-        # Attempt to get ABC widths (only for TrueType)
-        abc = ABC()
-        if gdi32.GetCharABCWidthsW(
-            self._dc,
-            ord(text), ord(text), byref(abc)
-        ):
-            width = abc.abcB
-            lsb = abc.abcA
-            advance = abc.abcA + abc.abcB + abc.abcC
-
-        else:
-            width_buf = c_int()
-            gdi32.GetCharWidth32W(
-                self._dc, ord(text), ord(text), byref(width_buf))
-
-            width = width_buf.value
-            lsb = 0
-            advance = width
-
-        # Can't get glyph-specific dimensions, use whole line-height.
-        height = self._bitmap_height
-        image = self._get_image(text, width, height, lsb)
-
-        glyph = self.font.create_glyph(image)
-        glyph.set_bearings(-self.font.descent, lsb, advance)
-
-        if _debug_font:
-            _debug(f'{self}.render({text})')
-            _debug(f'abc.abcA = {abc.abcA}')
-            _debug(f'abc.abcB = {abc.abcB}')
-            _debug(f'abc.abcC = {abc.abcC}')
-            _debug(f'width = {width}')
-            _debug(f'height = {height}')
-            _debug(f'lsb = {lsb}')
-            _debug(f'advance = {advance}')
-            _debug_image(image, f'glyph_{text}')
-            _debug_image(self.font.textures[0], f'tex_{text}')
-
-        return glyph
-
-    def _get_image(self, text: str, width: int, height: int, lsb: int) -> pyglet.image.ImageData:
-        # There's no such thing as a greyscale bitmap format in GDI.  We can
-        # create an 8-bit palette bitmap with 256 shades of grey, but
-        # unfortunately antialiasing will not work on such a bitmap.  So, we
-        # use a 32-bit bitmap and use the red channel as OpenGL's alpha.
-
-        gdi32.SelectObject(self._dc, self._bitmap)
-        gdi32.SelectObject(self._dc, self.font.hfont)
-        gdi32.SetBkColor(self._dc, 0x0)
-        gdi32.SetTextColor(self._dc, 0x00ffffff)
-        gdi32.SetBkMode(self._dc, OPAQUE)
-
-        # Draw to DC
-        user32.FillRect(self._dc, byref(self._bitmap_rect), self._black)
-        gdi32.ExtTextOutA(
-            self._dc, -lsb, 0, 0, None, text,
-            len(text), None)
-
-        gdi32.GdiFlush()
-
-        # Create glyph object and copy bitmap data to texture
-        image = pyglet.image.ImageData(
-            width, height,
-            'AXXX', self._bitmap_data, self._bitmap_rect.right * 4)
-
-        return image
-
-    def _create_bitmap(self, width: int, height: int) -> None:
-        self._black = gdi32.GetStockObject(BLACK_BRUSH)
-        self._white = gdi32.GetStockObject(WHITE_BRUSH)
-
-        if self._dc:
-            gdi32.ReleaseDC(self._dc)
-        if self._bitmap:
-            gdi32.DeleteObject(self._bitmap)
-
-        pitch = width * 4
-        data = POINTER(c_byte * (height * pitch))()
-        info = BITMAPINFO()
-        info.bmiHeader.biSize = sizeof(info.bmiHeader)
-        info.bmiHeader.biWidth = width
-        info.bmiHeader.biHeight = height
-        info.bmiHeader.biPlanes = 1
-        info.bmiHeader.biBitCount = 32
-        info.bmiHeader.biCompression = BI_RGB
-
-        self._dc = gdi32.CreateCompatibleDC(None)
-        self._bitmap = gdi32.CreateDIBSection(
-            None,
-            byref(info), DIB_RGB_COLORS, byref(data), None,
-            0)
-        # Spookiness: the above line causes a "not enough storage" error,
-        # even though that error cannot be generated according to docs,
-        # and everything works fine anyway.  Call SetLastError to clear it.
-        kernel32.SetLastError(0)
-
-        self._bitmap_data = data.contents
-        self._bitmap_rect = RECT()
-        self._bitmap_rect.left = 0
-        self._bitmap_rect.right = width
-        self._bitmap_rect.top = 0
-        self._bitmap_rect.bottom = height
-        self._bitmap_height = height
-
-        if _debug_font:
-            _debug(f'{self}._create_dc({width}, {height})')
-            _debug(f'_dc = {self._dc}')
-            _debug(f'_bitmap = {self._bitmap}')
-            _debug(f'pitch = {pitch}')
-            _debug(f'info.bmiHeader.biSize = {info.bmiHeader.biSize}')
-
-
-class Win32Font(base.Font):
-
-    glyph_renderer_class = GDIGlyphRenderer
-
-    def __init__(
-            self,
-            name: str, size: int,
-            bold: bool = False, italic: bool = False, stretch: bool = False,
-            dpi: Optional[float] = None
-    ):
-        super(Win32Font, self).__init__()
-
-        self.logfont = self.get_logfont(name, size, bold, italic, dpi)
-        self.hfont = gdi32.CreateFontIndirectW(byref(self.logfont))
-
-        # Create a dummy DC for coordinate mapping
-        with device_context(None) as dc:
-            metrics = TEXTMETRIC()
-            gdi32.SelectObject(dc, self.hfont)
-            gdi32.GetTextMetricsA(dc, byref(metrics))
-            self.ascent = metrics.tmAscent
-            self.descent = -metrics.tmDescent
-            self.max_glyph_width = metrics.tmMaxCharWidth
-
-    def __del__(self):
-        gdi32.DeleteObject(self.hfont)
-
-    @staticmethod
-    def get_logfont(name: str, size: float, bold: bool, italic: bool, dpi: Optional[float] = None) -> LOGFONTW:
-        """Get a raw Win32 :py:class:`.LOGFONTW` struct for the given arguments.
-
-        Args:
-            name: The name of the font
-            size: The font size in points
-            bold: Whether to request the font as bold
-            italic: Whether to request the font as italic
-            dpi: The screen dpi
-
-        Returns:
-            LOGFONTW: a ctypes binding of a Win32 LOGFONTW struct
-        """
-
-        # Create a dummy DC for coordinate mapping
-        with device_context(None) as dc:
-
-            # Default to 96 DPI unless otherwise specified
-            if dpi is None:
-                dpi = 96
-            log_pixels_y = dpi
-
-            # Create LOGFONTW font description struct
-            logfont = LOGFONTW()
-
-            # Convert point size to actual device pixels
-            logfont.lfHeight = int(-size * log_pixels_y // 72)
-
-            # Configure the LOGFONTW's font properties
-            if bold:
-                logfont.lfWeight = FW_BOLD
-            else:
-                logfont.lfWeight = FW_NORMAL
-            logfont.lfItalic = italic
-            logfont.lfFaceName = name
-            logfont.lfQuality = ANTIALIASED_QUALITY
-
-        return logfont
-
-    @classmethod
-    def have_font(cls, name) -> bool:
-        # [ ] add support for loading raster fonts
-        return win32query.have_font(name)
-
-    @classmethod
-    def add_font_data(cls, data):
-        numfonts = c_uint32()
-        _handle = gdi32.AddFontMemResourceEx(data, len(data), 0, byref(numfonts))
-
-        # None means a null handle was returned, ie something went wrong
-        if _handle is None:
-            raise ctypes.WinError()
-
-
-
-# --- GDI+ font rendering ---
-from pyglet.image.codecs.gdiplus import PixelFormat32bppARGB, gdiplus, Rect
-from pyglet.image.codecs.gdiplus import ImageLockModeRead, BitmapData
-
 DriverStringOptionsCmapLookup = 1
 DriverStringOptionsRealizedAdvance = 4
 TextRenderingHintAntiAlias = 4
 TextRenderingHintAntiAliasGridFit = 3
 
+FontStyleBold = 1
+FontStyleItalic = 2
+UnitPixel = 2
+UnitPoint = 3
+
 StringFormatFlagsDirectionRightToLeft = 0x00000001
 StringFormatFlagsDirectionVertical = 0x00000002
 StringFormatFlagsNoFitBlackBox = 0x00000004
@@ -309,33 +40,54 @@ StringFormatFlagsNoWrap = 0x00001000
 StringFormatFlagsLineLimit = 0x00002000
 StringFormatFlagsNoClip = 0x00004000
 
+FontFamilyNotFound = 14
+
+
+_debug_font = pyglet.options["debug_font"]
+
 
 class Rectf(ctypes.Structure):
     _fields_ = [
-        ('x', ctypes.c_float),
-        ('y', ctypes.c_float),
-        ('width', ctypes.c_float),
-        ('height', ctypes.c_float),
+        ("x", ctypes.c_float),
+        ("y", ctypes.c_float),
+        ("width", ctypes.c_float),
+        ("height", ctypes.c_float),
     ]
 
 
-class GDIPlusGlyphRenderer(Win32GlyphRenderer):
-    def __del__(self):
+class GDIPlusGlyphRenderer(base.GlyphRenderer):
+    def __init__(self, font: GDIPlusFont) -> None:
+        self._bitmap = None
+        self._dc = None
+        self._bitmap_rect = None
+        super().__init__(font)
+        self.font = font
+
+        # Pessimistically round up width and height to 4 byte alignment
+        width = font.max_glyph_width
+        height = font.ascent - font.descent
+        width = (width | 0x3) + 1
+        height = (height | 0x3) + 1
+        self._create_bitmap(width, height)
+
+        gdi32.SelectObject(self._dc, self.font.hfont)
+
+    def __del__(self) -> None:
         try:
             if self._matrix:
-                res = gdiplus.GdipDeleteMatrix(self._matrix)
+                gdiplus.GdipDeleteMatrix(self._matrix)
             if self._brush:
-                res = gdiplus.GdipDeleteBrush(self._brush)
+                gdiplus.GdipDeleteBrush(self._brush)
             if self._graphics:
-                res = gdiplus.GdipDeleteGraphics(self._graphics)
+                gdiplus.GdipDeleteGraphics(self._graphics)
             if self._bitmap:
-                res = gdiplus.GdipDisposeImage(self._bitmap)
+                gdiplus.GdipDisposeImage(self._bitmap)
             if self._dc:
-                res = user32.ReleaseDC(0, self._dc)
-        except:
+                user32.ReleaseDC(0, self._dc)
+        except Exception:  # noqa: S110, BLE001
             pass
 
-    def _create_bitmap(self, width, height):
+    def _create_bitmap(self, width: int, height: int) -> None:
         self._data = (BYTE * (4 * width * height))()
         self._bitmap = ctypes.c_void_p()
         self._format = PixelFormat32bppARGB
@@ -366,7 +118,7 @@ class GDIPlusGlyphRenderer(Win32GlyphRenderer):
 
         self._bitmap_height = height
 
-    def render(self, text):
+    def render(self, text: str) -> Glyph:
         ch = ctypes.create_unicode_buffer(text)
         len_ch = len(text)
 
@@ -425,8 +177,8 @@ class GDIPlusGlyphRenderer(Win32GlyphRenderer):
 
         # GDI functions only work for a single character so we transform
         # grapheme \r\n into \r
-        if text == '\r\n':
-            text = '\r'
+        if text == "\r\n":
+            text = "\r"
 
         # XXX END HACK HACK HACK
 
@@ -438,7 +190,7 @@ class GDIPlusGlyphRenderer(Win32GlyphRenderer):
         # For special unicode characters it may be comprised of 2+ codepoints. Get the width/lsb of each.
         # Function only works on TTF fonts.
         for codepoint in [ord(c) for c in text]:
-            if gdi32.GetCharABCWidthsW(self._dc, codepoint, codepoint, byref(abc)):
+            if gdi32.GetCharABCWidthsW(self._dc, codepoint, codepoint, ctypes.byref(abc)):
                 lsb += abc.abcA
                 width += abc.abcB
 
@@ -482,20 +234,20 @@ class GDIPlusGlyphRenderer(Win32GlyphRenderer):
         bitmap_data = BitmapData()
         gdiplus.GdipBitmapLockBits(
             self._bitmap,
-            byref(self._rect), ImageLockModeRead, self._format,
-            byref(bitmap_data))
+            ctypes.byref(self._rect), ImageLockModeRead, self._format,
+            ctypes.byref(bitmap_data))
 
         # Create buffer for RawImage
-        buffer = create_string_buffer(
+        buffer = ctypes.create_string_buffer(
             bitmap_data.Stride * bitmap_data.Height)
-        memmove(buffer, bitmap_data.Scan0, len(buffer))
+        ctypes.memmove(buffer, bitmap_data.Scan0, len(buffer))
 
         # Unlock data
-        gdiplus.GdipBitmapUnlockBits(self._bitmap, byref(bitmap_data))
+        gdiplus.GdipBitmapUnlockBits(self._bitmap, ctypes.byref(bitmap_data))
 
         image = pyglet.image.ImageData(
             width, height,
-            'BGRA', buffer, -bitmap_data.Stride)
+            "BGRA", buffer, -bitmap_data.Stride)
 
         glyph = self.font.create_glyph(image)
         # Only pass negative LSB info
@@ -504,28 +256,106 @@ class GDIPlusGlyphRenderer(Win32GlyphRenderer):
         return glyph
 
 
-FontStyleBold = 1
-FontStyleItalic = 2
-UnitPixel = 2
-UnitPoint = 3
+class Win32Font(base.Font):
+    glyph_renderer_class = GDIPlusGlyphRenderer
+
+    def __init__(
+            self,
+            name: str, size: float,
+            bold: bool = False, italic: bool = False, stretch: bool = False,
+            dpi: int | None = None,
+    ) -> None:
+        super().__init__()
+
+        self.logfont = self.get_logfont(name, size, bold, italic, dpi)
+        self.hfont = gdi32.CreateFontIndirectW(ctypes.byref(self.logfont))
+
+        # Create a dummy DC for coordinate mapping
+        with device_context(None) as dc:
+            metrics = TEXTMETRIC()
+            gdi32.SelectObject(dc, self.hfont)
+            gdi32.GetTextMetricsA(dc, ctypes.byref(metrics))
+            self.ascent = metrics.tmAscent
+            self.descent = -metrics.tmDescent
+            self.max_glyph_width = metrics.tmMaxCharWidth
+
+    @staticmethod
+    def get_logfont(name: str, size: float, bold: bool, italic: bool, dpi: int | None = None) -> LOGFONTW:
+        """Get a raw Win32 :py:class:`.LOGFONTW` struct for the given arguments.
+
+        Args:
+            name: The name of the font
+            size: The font size in points
+            bold: Whether to request the font as bold
+            italic: Whether to request the font as italic
+            dpi: The screen dpi
+
+        Returns:
+            LOGFONTW: a ctypes binding of a Win32 LOGFONTW struct
+        """
+
+        # Create a dummy DC for coordinate mapping
+        with device_context(None) as dc:
+
+            # Default to 96 DPI unless otherwise specified
+            if dpi is None:
+                dpi = 96
+            log_pixels_y = dpi
+
+            # Create LOGFONTW font description struct
+            logfont = LOGFONTW()
+
+            # Convert point size to actual device pixels
+            logfont.lfHeight = int(-size * log_pixels_y // 72)
+
+            # Configure the LOGFONTW's font properties
+            if bold:
+                logfont.lfWeight = FW_BOLD
+            else:
+                logfont.lfWeight = FW_NORMAL
+            logfont.lfItalic = italic
+            logfont.lfFaceName = name
+            logfont.lfQuality = ANTIALIASED_QUALITY
+
+        return logfont
+
+
+def _get_font_families(font_collection: ctypes.c_void_p) -> Sequence[ctypes.c_void_p]:
+    num_count = ctypes.c_int()
+    gdiplus.GdipGetFontCollectionFamilyCount(
+        font_collection, ctypes.byref(num_count))
+    gpfamilies = (ctypes.c_void_p * num_count.value)()
+    numFound = ctypes.c_int()
+    gdiplus.GdipGetFontCollectionFamilyList(
+        font_collection, num_count, gpfamilies, ctypes.byref(numFound))
+
+    return gpfamilies
+
+def _font_exists_in_collection(font_collection: ctypes.c_void_p, name: str) -> bool:
+    font_name = ctypes.create_unicode_buffer(32)
+    for gpfamily in _get_font_families(font_collection):
+        gdiplus.GdipGetFamilyName(gpfamily, font_name, "\0")
+        if font_name.value == name:
+            return True
+
+    return False
 
 
 class GDIPlusFont(Win32Font):
-    glyph_renderer_class = GDIPlusGlyphRenderer
+    glyph_renderer_class: ClassVar[type[base.GlyphRenderer]] = GDIPlusGlyphRenderer
 
-    _private_fonts = None
+    _private_collection: ctypes.c_void_p | None = None
+    _system_collection: ctypes.c_void_p | None = None
 
-    _default_name = 'Arial'
+    _default_name = "Arial"
 
-    def __init__(self, name, size, bold=False, italic=False, stretch=False, dpi=None):
+    def __init__(self, name: str, size: float, bold: bool=False, italic: bool=False, stretch: bool=False,
+                 dpi: int | None=None) -> None:
         if not name:
             name = self._default_name
 
-        # assert type(bold) is bool, "Only a boolean value is supported for bold in the current font renderer."
-        # assert type(italic) is bool, "Only a boolean value is supported for bold in the current font renderer."
-
         if stretch:
-            warnings.warn("The current font render does not support stretching.")
+            warnings.warn("The current font render does not support stretching.")  # noqa: B028
 
         super().__init__(name, size, bold, italic, stretch, dpi)
 
@@ -541,13 +371,13 @@ class GDIPlusFont(Win32Font):
         name = ctypes.c_wchar_p(name)
 
         # Look in private collection first:
-        if self._private_fonts:
-            gdiplus.GdipCreateFontFamilyFromName(name, self._private_fonts, ctypes.byref(family))
+        if self._private_collection:
+            gdiplus.GdipCreateFontFamilyFromName(name, self._private_collection, ctypes.byref(family))
 
         # Then in system collection:
         if not family:
             if _debug_font:
-                print(f"Warning: Font '{name}' was not found. Defaulting to: {self._default_name}")
+                print(f"Warning: Font '{name}' was not found. Defaulting to: {self._default_name}") 
 
             gdiplus.GdipCreateFontFamilyFromName(name, None, ctypes.byref(family))
 
@@ -574,41 +404,41 @@ class GDIPlusFont(Win32Font):
         gdiplus.GdipDeleteFontFamily(family)
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self._name
 
-    def __del__(self):
-        super(GDIPlusFont, self).__del__()
+    def __del__(self) -> None:
+        gdi32.DeleteObject(self.hfont)
         gdiplus.GdipDeleteFont(self._gdipfont)
 
     @classmethod
-    def add_font_data(cls, data):
-        super(GDIPlusFont, cls).add_font_data(data)
+    def add_font_data(cls: type[GDIPlusFont], data: bytes) -> None:
+        numfonts = ctypes.c_uint32()
+        _handle = gdi32.AddFontMemResourceEx(data, len(data), 0, ctypes.byref(numfonts))
 
-        if not cls._private_fonts:
-            cls._private_fonts = ctypes.c_void_p()
+        # None means a null handle was returned, ie something went wrong
+        if _handle is None:
+            raise ctypes.WinError()
+
+        if not cls._private_collection:
+            cls._private_collection = ctypes.c_void_p()
             gdiplus.GdipNewPrivateFontCollection(
-                ctypes.byref(cls._private_fonts))
-        gdiplus.GdipPrivateAddMemoryFont(cls._private_fonts, data, len(data))
+                ctypes.byref(cls._private_collection))
+        gdiplus.GdipPrivateAddMemoryFont(cls._private_collection, data, len(data))
 
     @classmethod
-    def have_font(cls, name):
+    def have_font(cls: type[GDIPlusFont], name: str) -> bool:
+        # Enumerate the private collection fonts first, as those are most likely to be used.
+        if cls._private_collection and _font_exists_in_collection(cls._private_collection, name):
+            return True
+
+        # Instead of enumerating all fonts on the system, as there can potentially be thousands, attempt to create
+        # the font family with the name. If it does not error (0), then it exists in the system.
         family = ctypes.c_void_p()
+        status = gdiplus.GdipCreateFontFamilyFromName(name, None, ctypes.byref(family))
+        if status == 0:
+            # Delete temp family to prevent leak.
+            gdiplus.GdipDeleteFontFamily(family)
+            return True
 
-        # Look in private collection first:
-        num_count = ctypes.c_int()
-        gdiplus.GdipGetFontCollectionFamilyCount(
-            cls._private_fonts, ctypes.byref(num_count))
-        gpfamilies = (ctypes.c_void_p * num_count.value)()
-        numFound = ctypes.c_int()
-        gdiplus.GdipGetFontCollectionFamilyList(
-            cls._private_fonts, num_count, gpfamilies, ctypes.byref(numFound))
-
-        font_name = ctypes.create_unicode_buffer(32)
-        for gpfamily in gpfamilies:
-            gdiplus.GdipGetFamilyName(gpfamily, font_name, '\0')
-            if font_name.value == name:
-                return True
-
-        # Else call parent class for system fonts
-        return super(GDIPlusFont, cls).have_font(name)
+        return False
