@@ -1,6 +1,5 @@
 import os
 import time
-import fcntl
 import ctypes
 import warnings
 
@@ -17,77 +16,20 @@ import pyglet
 
 from .evdev_constants import *
 from pyglet.app.xlib import XlibSelectDevice
+from pyglet.libs.ioctl import _IOR, _IOR_str, _IOR_len, _IOW
 from pyglet.input.base import Device, RelativeAxis, AbsoluteAxis, Button, Joystick, Controller
 from pyglet.input.base import DeviceOpenException, ControllerManager
 from pyglet.input.controller import get_mapping, Relation, create_guid
 
 try:
-    from os import readv as _os_readv
+    from os import readv as _readv
 except ImportError:
     # Workaround for missing os.readv in PyPy
     c = pyglet.lib.load_library('c')
-    def _os_readv(fd, buffers):
+
+    def _readv(fd, buffers):
         return c.read(fd, buffers, 3072)
 
-_IOC_NRBITS = 8
-_IOC_TYPEBITS = 8
-_IOC_SIZEBITS = 14
-_IOC_DIRBITS = 2
-
-_IOC_NRSHIFT = 0
-_IOC_TYPESHIFT = (_IOC_NRSHIFT + _IOC_NRBITS)
-_IOC_SIZESHIFT = (_IOC_TYPESHIFT + _IOC_TYPEBITS)
-_IOC_DIRSHIFT = (_IOC_SIZESHIFT + _IOC_SIZEBITS)
-
-_IOC_NONE = 0
-_IOC_WRITE = 1
-_IOC_READ = 2
-
-
-def _IOC(dir, type, nr, size):
-    return ((dir << _IOC_DIRSHIFT) |
-            (type << _IOC_TYPESHIFT) |
-            (nr << _IOC_NRSHIFT) |
-            (size << _IOC_SIZESHIFT))
-
-
-def _IOR(type, nr, struct):
-    request = _IOC(_IOC_READ, ord(type), nr, ctypes.sizeof(struct))
-
-    def f(fileno):
-        buffer = struct()
-        fcntl.ioctl(fileno, request, buffer)
-        return buffer
-
-    return f
-
-
-def _IOR_len(type, nr):
-    def f(fileno, buffer):
-        request = _IOC(_IOC_READ, ord(type), nr, ctypes.sizeof(buffer))
-        fcntl.ioctl(fileno, request, buffer)
-        return buffer
-
-    return f
-
-
-def _IOR_str(type, nr):
-    g = _IOR_len(type, nr)
-
-    def f(fileno, length=256):
-        return g(fileno, ctypes.create_string_buffer(length)).value
-
-    return f
-
-
-def _IOW(type, nr):
-
-    def f(fileno, buffer):
-        request = _IOC(_IOC_WRITE, ord(type), nr, ctypes.sizeof(buffer))
-        fcntl.ioctl(fileno, request, buffer)
-
-    return f
-
 
 # Structures from /linux/blob/master/include/uapi/linux/input.h
 
@@ -221,7 +163,7 @@ EVIOCGID = _IOR('E', 0x02, InputID)
 EVIOCGNAME = _IOR_str('E', 0x06)
 EVIOCGPHYS = _IOR_str('E', 0x07)
 EVIOCGUNIQ = _IOR_str('E', 0x08)
-EVIOCSFF = _IOW('E', 0x80)
+EVIOCSFF = _IOW('E', 0x80, FFEvent)
 
 
 def EVIOCGBIT(fileno, ev, buffer):
@@ -415,7 +357,7 @@ class EvdevDevice(XlibSelectDevice, Device):
             return
 
         try:
-            bytes_read = _os_readv(self._fileno, self._event_buffer)
+            bytes_read = _readv(self._fileno, self._event_buffer)
         except OSError:
             self.close()
             return
@@ -439,6 +381,8 @@ class FFController(Controller):
     _play_strong_event = None
     _stop_strong_event = None
 
+    device: EvdevDevice
+
     def open(self, window=None, exclusive=False):
         super().open(window, exclusive)
         self._fileno = self.device.fileno()
@@ -455,6 +399,7 @@ class FFController(Controller):
         self._stop_strong_event = InputEvent(Timeval(), EV_FF, self._strong_effect.id, 0)
 
     def rumble_play_weak(self, strength=1.0, duration=0.5):
+        assert self._fileno, "Controller must be opened first."
         effect = self._weak_effect
         effect.u.ff_rumble_effect.weak_magnitude = int(max(min(1.0, strength), 0) * 0xFFFF)
         effect.ff_replay.length = int(duration * 1000)
@@ -462,6 +407,7 @@ class FFController(Controller):
         self.device.ff_upload_effect(self._play_weak_event)
 
     def rumble_play_strong(self, strength=1.0, duration=0.5):
+        assert self._fileno, "Controller must be opened first."
         effect = self._strong_effect
         effect.u.ff_rumble_effect.strong_magnitude = int(max(min(1.0, strength), 0) * 0xFFFF)
         effect.ff_replay.length = int(duration * 1000)
@@ -469,9 +415,11 @@ class FFController(Controller):
         self.device.ff_upload_effect(self._play_strong_event)
 
     def rumble_stop_weak(self):
+        assert self._fileno, "Controller must be opened first."
         self.device.ff_upload_effect(self._stop_weak_event)
 
     def rumble_stop_strong(self):
+        assert self._fileno, "Controller must be opened first."
         self.device.ff_upload_effect(self._stop_strong_event)
 
 
